<?
class Sonos extends IPSModule
{
    
    public function __construct($InstanceID)
    {
        //Never delete this line!
        parent::__construct($InstanceID);
        
        //These lines are parsed on Symcon Startup or Instance creation
        //You cannot use variables here. Just static values.
        $this->RegisterPropertyString("IPAddress", "");
        $this->RegisterPropertyInteger("DefaultVolume", 15);
        $this->RegisterPropertyBoolean("GroupCoordinator", false);
        $this->RegisterPropertyBoolean("GroupForcing", false);
        $this->RegisterPropertyString("FavoriteStation",  "");
        $this->RegisterPropertyString("WebFrontStations", "<all>");
        $this->RegisterPropertyString("RINCON", "");
        $this->RegisterPropertyInteger("LCNDisplayId", 0);
        $this->RegisterPropertyInteger("LCNDisplayZeile", 0);
        
    }
    
    public function ApplyChanges()
    {
        $ipAddress = $this->ReadPropertyString("IPAddress");
        if ($ipAddress){
            $curl = curl_init();
            curl_setopt_array($curl, array(
            CURLOPT_RETURNTRANSFER => 1,
            CURLOPT_URL => 'http://'.$ipAddress.':1400/xml/device_description.xml'
            ));

            $result = curl_exec($curl);

            if(!curl_exec($curl))  die('Error: "' . curl_error($curl) . '" - Code: ' . curl_errno($curl));
        }

        //Never delete this line!
        parent::ApplyChanges();
        
        $this->RegisterProfileIntegerEx("Status.SONOS", "Information", "", "", Array(
                                             Array(0, "Prev", "", -1),
                                             Array(1, "Play", "", -1),
                                             Array(2, "Pause", "", -1),
                                             Array(3, "Stop", "", -1),
                                             Array(4, "Next", "", -1)
        ));
        $this->RegisterProfileInteger("Volume.SONOS", "Intensity", "", " %", 0, 100, 1);
        $this->RegisterProfileIntegerEx("Schalter.SONOS", "Information", "", "", Array(
                                             Array(0, "Off", "", 0xFF0000),
                                             Array(1, "On",  "", 0x00FF00)
        ));
        
        //Build Radio Station Associations according to user settings
        include(__DIR__ . "/radio_stations.php");
        $Associations          = Array();
        $AvailableStations     = get_available_stations();
        $WebFrontStations      = $this->ReadPropertyString("WebFrontStations");
        $WebFrontStationsArray = array_map("trim", explode(",", $WebFrontStations));
        $FavoriteStation       = $this->ReadPropertyString("FavoriteStation");
        $Value                 = 1;
        
        foreach ( $AvailableStations as $key => $val ) {
            if (in_array( $val['name'], $WebFrontStationsArray) || $WebFrontStations === "<alle>" || $WebFrontStations === "<all>" ) {
                if  ( $val['name'] === $FavoriteStation ){
                    $Color = 0xFCEC00;
                } else {
                    $Color = -1;
                }
                $Associations[] = Array($Value++, $val['name'], "", $Color);
            }
        }
        
        if(IPS_VariableProfileExists("Radio.SONOS"))
            IPS_DeleteVariableProfile("Radio.SONOS");
        
        $this->RegisterProfileIntegerEx("Radio.SONOS", "Speaker", "", "", $Associations);
        
        // Build Group Associations according Sonos Instance settings
        $allSonosInstances = IPS_GetInstanceListByModuleID("{F6F3A773-F685-4FD2-805E-83FD99407EE8}");
        $GroupAssociations = Array(Array(0, "none", "", -1));
        
        foreach($allSonosInstances as $key=>$SonosID) {
            if (IPS_GetProperty($SonosID, "GroupCoordinator"))
                $GroupAssociations[] = Array($SonosID, IPS_GetName($SonosID), "", -1);
        }
        
        if(IPS_VariableProfileExists("Groups.SONOS")) 
            IPS_DeleteVariableProfile("Groups.SONOS");
        
        $this->RegisterProfileIntegerEx("Groups.SONOS", "Network", "", "", $GroupAssociations);
        
        $this->RegisterVariableInteger("Status", "Status", "Status.SONOS");
        $this->EnableAction("Status");
        $this->RegisterVariableInteger("Volume", "Volume", "Volume.SONOS");
        $this->EnableAction("Volume");
        $this->RegisterVariableInteger("Radio", "Radio", "Radio.SONOS");
        $this->EnableAction("Radio");
        $this->RegisterVariableInteger("Mute", "Mute", "Schalter.SONOS");
        $this->EnableAction("Mute");
        $this->RegisterVariableString("nowPlaying", "nowPlaying", "");
        
        $links = Array();
        foreach( IPS_GetLinkList() as $key=>$LinkID ){
            $links[] =  Array( ('LinkID') => $LinkID, ('TargetID') =>  IPS_GetLink($LinkID)['TargetID'] );
        }
        
        // add group volume and  if group coordinator
        if ( $this->ReadPropertyBoolean("GroupCoordinator")){
            $this->RegisterVariableString("GroupMembers", "GroupMembers", "");
            $this->RegisterVariableInteger("GroupVolume", "GroupVolume", "Volume.SONOS");
            $this->EnableAction("GroupVolume");
            
            $vid = @$this->GetIDForIdent("MemberOfGroup");
            if ($vid){
                // delete links to Variable
                foreach( $links as $key=>$value ){
                    if ( $value['TargetID'] === $vid )
                         IPS_DeleteLink($value['LinkID']);
                }
                $this->DisableAction("MemberOfGroup");
                $this->UnregisterVariable("MemberOfGroup");
            }
        }else{
            $vid = @$this->GetIDForIdent("GroupVolume");
            if ($vid){
                // delete links to Variable
                foreach( $links as $key=>$value ){
                  if ( $value['TargetID'] === $vid )
                    IPS_DeleteLink($value['LinkID']);
                }
                $this->DisableAction("GroupVolume");
                $this->UnregisterVariable("GroupVolume");
            }
            
            $vid = @$this->GetIDForIdent("GroupMembers");
            if ($vid){
                // delete links to Variable
                foreach( $links as $key=>$value ){
                  if ( $value['TargetID'] === $vid )
                      IPS_DeleteLink($value['LinkID']);
                }
                $this->UnregisterVariable("GroupMembers");
            }
            
            $this->RegisterVariableInteger("MemberOfGroup", "MemberOfGroup", "Groups.SONOS");
            $this->EnableAction("MemberOfGroup");
        }
        
        // Hide and unhide Grouping depending on GroupCoordinators
        if (sizeof($GroupAssociations) === 1){
            // hide MemberOfGroup
            foreach($allSonosInstances as $key=>$SonosID) {
                $GroupingID = @IPS_GetVariableIDByName("MemberOfGroup",$SonosID);
                if ($GroupingID){
                    IPS_SetHidden($GroupingID,true);
                }
            }
        }else{
            // unhide MemberOfGroup
            foreach($allSonosInstances as $key=>$SonosID) {
                $GroupingID = @IPS_GetVariableIDByName("MemberOfGroup",$SonosID);
                if ($GroupingID){
                    IPS_SetHidden($GroupingID,false);
                }
            }
        }
        
        // add scripts for regular status and grouping updates
         
        $statusScriptID = $this->RegisterScript("_updateStatus", "_updateStatus", '<?
include("../modules/SymconMisc/Sonos/sonos.php");

$ip = IPS_GetProperty(IPS_GetParent($_IPS["SELF"]), "IPAddress");

if (Sys_Ping($ip, 1000) == true) {

    $sonos = new PHPSonos($ip);

    $status = $sonos->GetTransportInfo();
    SetValueInteger(IPS_GetObjectIDByName("Mute", IPS_GetParent($_IPS["SELF"])), $sonos->GetMute());
    SetValueInteger(IPS_GetObjectIDByName("Volume", IPS_GetParent($_IPS["SELF"])), $sonos->GetVolume());

    $MemberOfGroupID = @IPS_GetObjectIDByName("MemberOfGroup", IPS_GetParent($_IPS["SELF"]));
    $MemberOfGroup = 0;
    if ($MemberOfGroupID)
    $MemberOfGroup = GetValueInteger($MemberOfGroupID);

    if ($MemberOfGroup){
        // If Sonos is member of a group, use values of Group Coordinator
        SetValueInteger(IPS_GetObjectIDByName("Status", IPS_GetParent($_IPS["SELF"])), GetValueInteger(IPS_GetObjectIDByName("Status", $MemberOfGroup)));
        $actuallyPlaying = GetValueString(IPS_GetObjectIDByName("nowPlaying", $MemberOfGroup));
        SetValueInteger(IPS_GetObjectIDByName("Radio", IPS_GetParent($_IPS["SELF"])), GetValueInteger(IPS_GetObjectIDByName("Radio", $MemberOfGroup)));
        }else{
        SetValueInteger(IPS_GetObjectIDByName("Status", IPS_GetParent($_IPS["SELF"])), $status);
        // Titelanzeige
        $currentStation = 0;

        if ( $status <> 1 ){
            // No title if not playing
            $actuallyPlaying = "";
            }else{
            $positionInfo = $sonos->GetPositionInfo();
            $mediaInfo    = $sonos->GetMediaInfo();

            if (strlen($positionInfo["streamContent"]) <> 0){
                $title = $mediaInfo["title"];
                $actuallyPlaying = $positionInfo["streamContent"];
                } else {
                $actuallyPlaying = utf8_decode($positionInfo["title"]." | ".$positionInfo["artist"]);
            }
            // start find current Radio in VariableProfile
            $Associations = IPS_GetVariableProfile("Radio.SONOS")["Associations"];

            if(isset($mediaInfo["title"])){
              foreach($Associations as $key=>$station) {
                  if( $station["Name"] == $mediaInfo["title"] ){
                      $currentStation = $Associations[$key]["Value"];
                  }
              }
            }
            // end find current Radio in VariableProfile
        }
        SetValueInteger(IPS_GetObjectIDByName("Radio", IPS_GetParent($_IPS["SELF"])), $currentStation);
    }

    $nowPlaying   = GetValueString(IPS_GetObjectIDByName("nowPlaying", IPS_GetParent($_IPS["SELF"])));

    if ($actuallyPlaying <> $nowPlaying) {
        SetValueString(IPS_GetObjectIDByName("nowPlaying", IPS_GetParent($_IPS["SELF"])), $actuallyPlaying);
        $DisplayId    = intval(IPS_GetProperty(IPS_GetParent($_IPS["SELF"]), "LCNDisplayId"));
        $DisplayZeile = IPS_GetProperty(IPS_GetParent($_IPS["SELF"]), "LCNDisplayZeile");

        if ($DisplayId != 0) {
            if (strlen($positionInfo["streamContent"]) != 0) {
                $actuallyPlaying = $actuallyPlaying."-".$title;
            }

            LCN_SendCommand($DisplayId, "GT", "DT" . $DisplayZeile . "1" . utf8_decode(substr($actuallyPlaying,  0, 12)));
            LCN_SendCommand($DisplayId, "GT", "DT" . $DisplayZeile . "2" . utf8_decode(substr($actuallyPlaying, 12, 12)));
            LCN_SendCommand($DisplayId, "GT", "DT" . $DisplayZeile . "3" . utf8_decode(substr($actuallyPlaying, 24, 12)));
            LCN_SendCommand($DisplayId, "GT", "DT" . $DisplayZeile . "4" . utf8_decode(substr($actuallyPlaying, 36, 12)));
            LCN_SendCommand($DisplayId, "GT", "DT" . $DisplayZeile . "5" . utf8_decode(substr($actuallyPlaying, 48, 12)));
        }

    }
}

// Set Group Volume
if(IPS_GetProperty(IPS_GetParent($_IPS["SELF"]), "GroupCoordinator")){
  $groupMembers        = GetValueString(IPS_GetObjectIDByName("GroupMembers", IPS_GetParent($_IPS["SELF"])));
  $groupMembersArray   = Array();
  if($groupMembers)
    $groupMembersArray = array_map("intval", explode(",",$groupMembers));
  $groupMembersArray[] = IPS_GetParent($_IPS["SELF"]);

  $GroupVolume = 0;
  foreach($groupMembersArray as $key=>$ID) {
    $GroupVolume += GetValueInteger(IPS_GetObjectIDByName("Volume", $ID));
  }
  
  SetValueInteger(IPS_GetObjectIDByName("GroupVolume", IPS_GetParent($_IPS["SELF"])), round($GroupVolume / sizeof($groupMembersArray)));
}
?>');

        IPS_SetHidden($statusScriptID,true);
        IPS_SetScriptTimer($statusScriptID, 5); 

        $groupingScriptID = $this->RegisterScript("_updateGrouping", "_updateGrouping", '<?
include("../modules/SymconMisc/Sonos/sonos.php");

$groupCoordinator = IPS_GetProperty(IPS_GetParent($_IPS["SELF"]), "GroupCoordinator");

// Nothing to do if Instance is Group Coordinator
if($groupCoordinator) exit;

$groupForcing      = IPS_GetProperty(IPS_GetParent($_IPS["SELF"]), "GroupForcing");
$rinconMapping     = Array();
$allSonosInstances = IPS_GetInstanceListByModuleID("{F6F3A773-F685-4FD2-805E-83FD99407EE8}");
$MemberOfGroupID   = @IPS_GetObjectIDByName("MemberOfGroup", IPS_GetParent($_IPS["SELF"]));
$MemberOfGroup     = 0;
if ($MemberOfGroupID)  $MemberOfGroup = GetValueInteger($MemberOfGroupID);

//ensure that all rincons are known
foreach($allSonosInstances as $key=>$SonosID) {
    $rincon = IPS_GetProperty($SonosID ,"RINCON");
    if (!$rincon){
        // Get RINCON
        // Not sure why, but when executed in ApplyChanges of module.php RINCON is not alway set
        $ipAddress = IPS_GetProperty($SonosID, "IPAddress");
        if ($ipAddress){
            $curl = curl_init();
            curl_setopt_array($curl, array(
            CURLOPT_RETURNTRANSFER => 1,
            CURLOPT_URL => "http://".$ipAddress.":1400/xml/device_description.xml"
            ));

            $result = curl_exec($curl);

            if(!curl_exec($curl)){
                continue;
            }

            $xmlr = new SimpleXMLElement($result);
            $rincon = str_replace ( "uuid:" , "" , $xmlr->device->UDN );
            IPS_SetProperty($SonosID, "RINCON", $rincon );
            IPS_ApplyChanges($SonosID);
        }
    }
    $rinconMapping[] = Array( ("ID") => $SonosID, ("RINCON") => $rincon );
    if ($SonosID === IPS_GetParent($_IPS["SELF"])) $ownRincon = $rincon;
}

$ipAddress = IPS_GetProperty(IPS_GetParent($_IPS["SELF"]), "IPAddress");

if (Sys_Ping($ipAddress, 1000) == true) {

    $sonos                    = new PHPSonos($ipAddress);
    $sonosZoneGroupAttributes = (new SimpleXMLElement($sonos->GetZoneGroupAttributes()))->children("s",true)->Body->children("u",true)->GetZoneGroupAttributesResponse->children();
    $sonosGroupCoordinator    = explode(":",$sonosZoneGroupAttributes->CurrentZoneGroupID)[0];
    //  $sonosGroupMembers        = explode(",",$sonosZoneGroupAttributes->CurrentZonePlayerUUIDsInGroup);

    foreach($rinconMapping as $key=>$value) {
        if($value["RINCON"] === $sonosGroupCoordinator ){
            $sonosGroupCoordinatorID = $value["ID"] ;
            break;
        }
    }

    // If groupCoordinator in Sonos = this instance --> set ID to 0 (does not belong to group)
    if ($sonosGroupCoordinatorID === IPS_GetParent($_IPS["SELF"]))  $sonosGroupCoordinatorID = 0;

    if ( $sonosGroupCoordinatorID !== $MemberOfGroup ){
        if($groupForcing){
            SNS_SetGroup(IPS_GetParent($_IPS["SELF"]),$MemberOfGroup);
            }else{
            SNS_SetGroup(IPS_GetParent($_IPS["SELF"]),$sonosGroupCoordinatorID);
        }
    }
}
?>');

        IPS_SetHidden($groupingScriptID,true);
        IPS_SetScriptTimer($groupingScriptID, 300); 

    }
    
    /**
    * This function will be available automatically after the module is imported with the module control.
    * Using the custom prefix this function will be callable from PHP and JSON-RPC through:
    *
    * SNS_Play($id);
    *
    */
    public function Play()
    {
        include(__DIR__ . "/sonos.php");
        (new PHPSonos($this->ReadPropertyString("IPAddress")))->Play();
    }
    
    public function Pause()
    {
        include(__DIR__ . "/sonos.php");
        (new PHPSonos($this->ReadPropertyString("IPAddress")))->Pause();
    }
    
    public function Previous()
    {
        include(__DIR__ . "/sonos.php");
        (new PHPSonos($this->ReadPropertyString("IPAddress")))->Previous();
    }
    
    public function Next()
    {
        include(__DIR__ . "/sonos.php");
        (new PHPSonos($this->ReadPropertyString("IPAddress")))->Next();
    }
    
    public function SetMute($mute)
    {
        include(__DIR__ . "/sonos.php");
        (new PHPSonos($this->ReadPropertyString("IPAddress")))->SetMute($mute);
    }
    
    
    public function SetVolume($volume)
    {
        include(__DIR__ . "/sonos.php");
        (new PHPSonos($this->ReadPropertyString("IPAddress")))->SetVolume($volume);
    }

    public function SetDefaultVolume()
    {
        $this->SetVolume($this->ReadPropertyInteger("DefaultVolume"));
    }
    
    public function SetRadio($radio)
    {
        include(__DIR__ . "/sonos.php");
        include(__DIR__ . "/radio_stations.php");
        (new PHPSonos($this->ReadPropertyString("IPAddress")))->SetRadio( get_station_url($radio), $radio);
        (new PHPSonos($this->ReadPropertyString("IPAddress")))->Play();
    }
    
    public function SetGroup($groupCoordinator)
    {
        // get variable of coordinator members to be updated
        if ($groupCoordinator){
            $groupMembersID = @IPS_GetObjectIDByIdent("GroupMembers",$groupCoordinator);
            $uri            = "x-rincon:".IPS_GetProperty($groupCoordinator ,"RINCON");
            }else{
            $groupMembersID = @IPS_GetObjectIDByIdent("GroupMembers",GetValue($this->GetIDForIdent("MemberOfGroup")));
            $uri            = "";
        }
        
        // update coordinator members
        SetValue($this->GetIDForIdent("MemberOfGroup"), $groupCoordinator);
        
        if($groupMembersID){
            $currentMembers = explode(",",GetValueString($groupMembersID));
            $currentMembers = array_filter($currentMembers, function($v) { return $v != ""; });
            $currentMembers = array_filter($currentMembers, function($v) { return $v != $this->InstanceID ; });
            if($groupCoordinator)
            $currentMembers[] = $this->InstanceID;
            
            SetValueString($groupMembersID,implode(",",$currentMembers));
        }
        
        // Set reelevant variables to hidden/unhidden
        if ($groupCoordinator){
            $hidden = true ;
            }else{
            $hidden = false ;
        }
        IPS_SetHidden($this->GetIDForIdent("nowPlaying"),$hidden);
        IPS_SetHidden($this->GetIDForIdent("Radio"),$hidden);
        IPS_SetHidden($this->GetIDForIdent("Status"),$hidden);
        
        include(__DIR__ . "/sonos.php");
        (new PHPSonos($this->ReadPropertyString("IPAddress")))->SetAVTransportURI($uri);
    }

    public function SetDefaultGroupVolume()
    {
        if (!$this->ReadPropertyBoolean("GroupCoordinator")) die("This function is only allowed for GroupCoordinators");

        $groupMembers        = GetValueString(IPS_GetObjectIDByName("GroupMembers",$this->InstanceID ));
        $groupMembersArray   = Array();
        if($groupMembers)
            $groupMembersArray = array_map("intval", explode(",",$groupMembers));
        $groupMembersArray[] = $this->InstanceID;

        foreach($groupMembersArray as $key=>$ID) {
          SNS_SetDefaultVolume($ID);
        }
    }


    public function ChangeGroupVolume($change)
    {
        if (!$this->ReadPropertyBoolean("GroupCoordinator")) die("This function is only allowed for GroupCoordinators");
     
        $groupMembers        = GetValueString(IPS_GetObjectIDByName("GroupMembers",$this->InstanceID ));
        $groupMembersArray   = Array();
        if($groupMembers)
            $groupMembersArray = array_map("intval", explode(",",$groupMembers));
        $groupMembersArray[] = $this->InstanceID;
            
        foreach($groupMembersArray as $key=>$ID) {
          $newVolume = (GetValueInteger(IPS_GetObjectIDByName("Volume",$ID)) + $change);
          if ($newVolume > 100){
              $newVolume = 100;
          }elseif($newVolume < 0){
              $newVolume = 0;
          } 
          SNS_SetVolume($ID, $newVolume );
        }
    }
    
    public function SetRadioFavorite()
    {
        $this->SetRadio($this->ReadPropertyString("FavoriteStation"));
    }
    
    public function Stop()
    {
        include(__DIR__ . "/sonos.php");
        (new PHPSonos($this->ReadPropertyString("IPAddress")))->Stop();
    }
    
    public function RequestAction($Ident, $Value)
    {
        switch($Ident) {
            case "GroupVolume":
              $this->ChangeGroupVolume($Value-GetValue($this->GetIDForIdent($Ident)));
              SetValue($this->GetIDForIdent($Ident), $Value);
              break;
            case "MemberOfGroup":
              $this->SetGroup($Value);
              break;
            case "Mute":
              $this->SetMute($Value);
              SetValue($this->GetIDForIdent($Ident), $Value);
              break;
            case "Radio":
              $this->SetRadio(IPS_GetVariableProfile("Radio.SONOS")['Associations'][$Value-1]['Name']);
              SetValue($this->GetIDForIdent($Ident), $Value);
              break;
            case "Status":
              switch($Value) {
                  case 0: //Prev
                    $this->Previous();
                    break;
                  case 1: //Play
                    $this->Play();
                    SetValue($this->GetIDForIdent($Ident), $Value);
                    break;
                  case 2: //Pause
                    $this->Pause();
                    SetValue($this->GetIDForIdent($Ident), $Value);
                    break;
                  case 3: //Stop
                    $this->Stop();
                    SetValue($this->GetIDForIdent($Ident), $Value);
                    break;
                  case 4: //Next
                    $this->Next();
                    break;
              }
              break;
            case "Volume":
              $this->SetVolume($Value);
              SetValue($this->GetIDForIdent($Ident), $Value);
              break;
            default:
              throw new Exception("Invalid ident");
        }
    }
    
    //Remove on next Symcon update
    protected function RegisterProfileInteger($Name, $Icon, $Prefix, $Suffix, $MinValue, $MaxValue, $StepSize) {
        
        if(!IPS_VariableProfileExists($Name)) {
            IPS_CreateVariableProfile($Name, 1);
            } else {
            $profile = IPS_GetVariableProfile($Name);
            if($profile['ProfileType'] != 1)
            throw new Exception("Variable profile type does not match for profile ".$Name);
        }
        
        IPS_SetVariableProfileIcon($Name, $Icon);
        IPS_SetVariableProfileText($Name, $Prefix, $Suffix);
        IPS_SetVariableProfileValues($Name, $MinValue, $MaxValue, $StepSize);
        
    }
    
    protected function RegisterProfileIntegerEx($Name, $Icon, $Prefix, $Suffix, $Associations) {
        if ( sizeof($Associations) === 0 ){
            $MinValue = 0;
            $MaxValue = 0;
            } else {
            $MinValue = $Associations[0][0];
            $MaxValue = $Associations[sizeof($Associations)-1][0];
        }
        
        $this->RegisterProfileInteger($Name, $Icon, $Prefix, $Suffix, $MinValue, $MaxValue, 0);
        
        foreach($Associations as $Association) {
            IPS_SetVariableProfileAssociation($Name, $Association[0], $Association[1], $Association[2], $Association[3]);
        }
        
    }
}
?>
